@@ -371,7 +371,7 @@ static int addallcpus(void)
 			errno, strerror(errno));
 		return -1;
 	}
-		
+
 	while (!feof(fp)) {
 		/* scan all intervals of online cpus */
 		scanned = fscanf(fp, "%u-%u", &start, &end);
@@ -414,7 +414,7 @@ static void *hotplughandler(void *UNUSED(unused))
 	struct udev *hotplug;
 	struct udev_monitor *monitor;
 	struct pollfd item;
-	
+
 	hotplug = udev_new();
 	if (!hotplug) {
 		eprint("Failed to create hotplug device\n");
@@ -496,10 +496,8 @@ int perf_init(void)
 		num -= 2;
 	}
 
-	if (num == 0) {
+	if (num == 0)
 		eprint("No crypto counters supported!\n");
-		return -1;
-	}
 
 	if (perf_load_counter_data())
 		return -1;
