@@ -23,19 +23,27 @@ PvArg *pv_arg_new(PvComponentType type, const gchar *path);
 void pv_arg_free(PvArg *arg);
 
 typedef enum pv_tristate {
-		  PV_NOT_SET = 0,
-		  PV_TRUE,
-		  PV_FALSE,
+	PV_NOT_SET = 0,
+	PV_TRUE,
+	PV_FALSE,
 } PvTristate;
+/* The value of PV_NOT_SET is not allowed to be changed */
+STATIC_ASSERT(PV_NOT_SET == 0)
+
+typedef struct {
+	gchar *pcf;
+	gchar *scf;
+	/* Add-secret requests do require CCK-extension secrets */
+	PvTristate enable_cck_extension_secret_enforcement;
+	PvTristate enable_dump;
+	PvTristate enable_pckmo;
+} PvControlFlagsArgs;
 
 typedef struct {
 	gint log_level;
 	gint no_verify;
 	gboolean offline;
-	gchar *pcf;
-	gchar *scf;
-	PvTristate allow_dump;
-	PvTristate allow_pckmo;
+	PvControlFlagsArgs cf_args;
 	gchar *psw_addr; /* PSW address which will be used for the start of
 			  * the actual component (e.g. Linux kernel)
 			  */
