@@ -19,7 +19,7 @@
 typedef struct ipl_parameter_block IplParameterBlock;
 
 uint64_t pv_ipib_get_size(uint32_t num_comp);
-IplParameterBlock *pv_ipib_new(GSList *comps, const Buffer *hdr, GError **err);
+IplParameterBlock *pv_ipib_new(GSList *comps, const PvBuffer *hdr, GError **err);
 void pv_ipib_free(IplParameterBlock *ipib);
 
 WRAPPED_G_DEFINE_AUTOPTR_CLEANUP_FUNC(IplParameterBlock, pv_ipib_free)
