@@ -23,6 +23,9 @@
 
 #include "lib/zt_common.h"
 
+#include "libseckey/sk_openssl.h"
+#include "libseckey/sk_utilities.h"
+
 #include "utilities.h"
 
 #ifndef JSON_C_TO_STRING_NOSLASHESCAPE
@@ -397,6 +400,7 @@ int create_json_web_signature(const char *algorithm, bool b64, const char *kid,
 			      bool detached_payload, EVP_MD_CTX *md_ctx,
 			      char **jws)
 {
+	const struct sk_ec_curve_info *curve_info;
 	unsigned char *signature = NULL;
 	json_object *header_obj = NULL;
 	json_object *crit_obj = NULL;
@@ -531,8 +535,13 @@ int create_json_web_signature(const char *algorithm, bool b64, const char *kid,
 
 	switch (EVP_PKEY_id(pkey)) {
 	case EVP_PKEY_EC:
-		prime_len = ecc_get_curve_prime_length(EC_GROUP_get_curve_name(
-				EC_KEY_get0_group(EVP_PKEY_get0_EC_KEY(pkey))));
+		curve_info = SK_UTIL_ec_get_curve_info(
+				SK_OPENSSL_get_curve_from_ec_pkey(pkey));
+		if (curve_info == NULL) {
+			rc = -EINVAL;
+			goto out;
+		}
+		prime_len = curve_info->prime_len;
 
 		p = signature;
 		if (d2i_ECDSA_SIG(&ec_sig, &p, signature_len) == NULL) {
@@ -634,13 +643,12 @@ int verify_json_web_signature(const char *jws, const unsigned char *payload,
 	unsigned char *signature = NULL, *der = NULL, *sig = NULL;
 	json_object *header_obj = NULL, *b64_obj = NULL;
 	int der_len, rc, curve_nid = 0, digest_nid = 0;
-	struct ekmf_rsa_pss_params rsa_pss_params;
+	struct sk_rsa_pss_params rsa_pss_params;
 	bool b64 = true, rsa_pss = false;
 	EVP_MD_CTX *md_ctx = NULL;
 	EVP_PKEY_CTX *pctx = NULL;
 	ECDSA_SIG *ec_sig = NULL;
 	char *payload_b64 = NULL;
-	const EVP_MD *md = NULL;
 	BIGNUM *bn_r = NULL;
 	BIGNUM *bn_s = NULL;
 	const char *alg;
@@ -670,8 +678,7 @@ int verify_json_web_signature(const char *jws, const unsigned char *payload,
 	hdr_pld_len = ch - jws;
 
 	if (EVP_PKEY_id(pkey) == EVP_PKEY_EC) {
-		curve_nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(
-						EVP_PKEY_get0_EC_KEY(pkey)));
+		curve_nid = SK_OPENSSL_get_curve_from_ec_pkey(pkey);
 	}
 
 	alg = json_get_string(header_obj, "alg");
@@ -735,32 +742,18 @@ int verify_json_web_signature(const char *jws, const unsigned char *payload,
 		goto out;
 	}
 
-	md = EVP_get_digestbynid(digest_nid);
-	if (md == NULL) {
-		rc = -ENOTSUP;
-		goto out;
-	}
-
-	md_ctx = EVP_MD_CTX_new();
-	if (md_ctx == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	rc = EVP_DigestVerifyInit(md_ctx, &pctx, md, NULL, pkey);
-	if (rc != 1) {
-		rc = -EIO;
-		goto out;
-	}
-
+	memset(&rsa_pss_params, 0, sizeof(rsa_pss_params));
 	if (rsa_pss) {
 		rsa_pss_params.mgf_digest_nid = digest_nid;
 		rsa_pss_params.salt_len = RSA_PSS_SALTLEN_DIGEST;
-		rc = setup_rsa_pss_pkey_context(pctx, &rsa_pss_params);
-		if (rc != 0)
-			goto out;
 	}
 
+	rc = SK_OPENSSL_setup_sign_context(pkey, true, digest_nid,
+					   &rsa_pss_params,
+					   &md_ctx, &pctx, false);
+	if (rc != 0)
+		goto out;
+
 	switch (EVP_PKEY_id(pkey)) {
 	case EVP_PKEY_EC:
 		ec_sig = ECDSA_SIG_new();
@@ -1771,431 +1764,6 @@ char *get_http_header_value(const struct curl_slist *headers, const char *name)
 	return NULL;
 }
 
-struct ecc_curve_info {
-	int curve_nid;
-	enum {
-		ECC_TYPE_PRIME = 0,
-		ECC_TYPE_BRAINPOOL = 1,
-	} type;
-	size_t prime_bits;
-	size_t prime_len;
-	const char *curve_id;
-};
-
-static const struct ecc_curve_info ecc_curve_list[] = {
-	{ .curve_nid = NID_X9_62_prime192v1, .type = ECC_TYPE_PRIME,
-	  .prime_bits = 192, .prime_len = 24, .curve_id = "P-192" },
-	{ .curve_nid = NID_secp224r1,        .type = ECC_TYPE_PRIME,
-	  .prime_bits = 224, .prime_len = 28, .curve_id = "P-224" },
-	{ .curve_nid = NID_X9_62_prime256v1, .type = ECC_TYPE_PRIME,
-	  .prime_bits = 256, .prime_len = 32, .curve_id = "P-256" },
-	{ .curve_nid = NID_secp384r1,        .type = ECC_TYPE_PRIME,
-	  .prime_bits = 384, .prime_len = 48, .curve_id = "P-384" },
-	{ .curve_nid = NID_secp521r1,        .type = ECC_TYPE_PRIME,
-	  .prime_bits = 521, .prime_len = 66, .curve_id = "P-521" },
-	{ .curve_nid = NID_brainpoolP160r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 160, .prime_len = 20, .curve_id = "brainpoolP160r1" },
-	{ .curve_nid = NID_brainpoolP192r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 192, .prime_len = 24, .curve_id = "brainpoolP192r1" },
-	{ .curve_nid = NID_brainpoolP224r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 224, .prime_len = 28, .curve_id = "brainpoolP224r1" },
-	{ .curve_nid = NID_brainpoolP256r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 256, .prime_len = 32, .curve_id = "brainpoolP256r1" },
-	{ .curve_nid = NID_brainpoolP320r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 320, .prime_len = 40, .curve_id = "brainpoolP320r1" },
-	{ .curve_nid = NID_brainpoolP384r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 384, .prime_len = 48, .curve_id = "brainpoolP384r1" },
-	{ .curve_nid = NID_brainpoolP512r1,  .type = ECC_TYPE_BRAINPOOL,
-	  .prime_bits = 512, .prime_len = 64, .curve_id = "brainpoolP512r1" },
-};
-
-static const int ecc_curve_num =
-		sizeof(ecc_curve_list) / sizeof(struct ecc_curve_info);
-
-/**
- * Returns the prime bit length of the specified curve, or 0 if the curve
- * is not known.
- */
-size_t ecc_get_curve_prime_bits(int curve_nid)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].curve_nid == curve_nid)
-			return ecc_curve_list[i].prime_bits;
-	}
-	return 0;
-}
-
-/**
- * Returns the prime length in bytes of the specified curve, or 0 if the curve
- * is not known.
- */
-size_t ecc_get_curve_prime_length(int curve_nid)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].curve_nid == curve_nid)
-			return ecc_curve_list[i].prime_len;
-	}
-	return 0;
-}
-
-/**
- * Returns the textual curve ID of the specified curve, or NULL if the curve
- * is not known.
- */
-const char *ecc_get_curve_id(int curve_nid)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].curve_nid == curve_nid)
-			return ecc_curve_list[i].curve_id;
-	}
-	return NULL;
-}
-
-/**
- * Returns true if the specified curve is a Prime curve, false if not, or if
- * the curve is not known.
- */
-bool ecc_is_prime_curve(int curve_nid)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].curve_nid == curve_nid)
-			return ecc_curve_list[i].type == ECC_TYPE_PRIME;
-	}
-	return false;
-}
-
-/**
- * Returns true if the specified curve is a Brainpool curve, false if not, or if
- * the curve is not known.
- */
-bool ecc_is_brainpool_curve(int curve_nid)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].curve_nid == curve_nid)
-			return ecc_curve_list[i].type == ECC_TYPE_BRAINPOOL;
-	}
-	return false;
-}
-
-/**
- * Returns the nid of the curve of the specified curve ID, or 0 if the curve
- * is not known.
- */
-int ecc_get_curve_by_id(const char *curve_id)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (strcmp(ecc_curve_list[i].curve_id, curve_id) == 0)
-			return ecc_curve_list[i].curve_nid;
-	}
-	return 0;
-}
-
-/**
- * Returns the nid of the Prime curve by its specified prime bit size, or 0
- * if the curve is not knwon.
- */
-int ecc_get_prime_curve_by_prime_bits(size_t prime_bits)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].type == ECC_TYPE_PRIME &&
-		    ecc_curve_list[i].prime_bits == prime_bits)
-			return ecc_curve_list[i].curve_nid;
-	}
-	return 0;
-}
-
-/**
- * Returns the nid of the Brainpool curve by its specified prime bit size, or 0
- * if the curve is not knwon.
- */
-int ecc_get_brainpool_curve_by_prime_bits(size_t prime_bits)
-{
-	int i;
-
-	for (i = 0; i < ecc_curve_num; i++) {
-		if (ecc_curve_list[i].type == ECC_TYPE_BRAINPOOL &&
-		    ecc_curve_list[i].prime_bits == prime_bits)
-			return ecc_curve_list[i].curve_nid;
-	}
-	return 0;
-}
-
-/**
- * Calculates the y coordinate of a point on an EC curve using the x coordinate
- * and the y bit. x and y must be supplied by the caller with prime_len bytes.
- * On return y contains the calculated y coordinate.
- *
- * @param nid               the OpenSSL nid of the ECC curve used
- * @param prime_len         the length of the prime in bytes. This is also the
- *                          length of the x and y coordinates.
- * @param x                 the x coordinate as big endian binary number in
- *                          prime_len size
- * @param y_bit             the y-bit to identify which of the two possible
- *                          values for y should be used
- * @param y                 buffer to store the y coordinate as big endian
- *                          binary number in prime_len size.
- * @returns zero for success, a negative errno in case of an error:
- *          -EINVAL: a function parameter is invalid
- *          -ENOMEM: failed to allocate memory
- *          -EIO: OpenSSL failed to calculate the y coordinate
- *          -ENOENT: OpenSSL does not know/support the curve (nid)
- */
-int ecc_calculate_y_coordinate(int nid, size_t prime_len,
-			       const unsigned char *x, int y_bit,
-			       unsigned char *y)
-{
-	EC_GROUP *group = NULL;
-	EC_POINT *point = NULL;
-	BIGNUM *bn_x = NULL;
-	BIGNUM *bn_y = NULL;
-	BN_CTX *ctx = NULL;
-	int rc = 0;
-
-	if (x == NULL || y == NULL)
-		return -EINVAL;
-
-	bn_x = BN_bin2bn(x, prime_len, NULL);
-	if (bn_x == NULL) {
-		rc = -EIO;
-		goto out;
-	}
-
-	group = EC_GROUP_new_by_curve_name(nid);
-	if (group == NULL) {
-		rc = -ENOENT;
-		goto out;
-	}
-
-	point = EC_POINT_new(group);
-	if (point == NULL) {
-		rc = -EIO;
-		goto out;
-	}
-
-	bn_y = BN_new();
-	if (bn_y == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	ctx = BN_CTX_new();
-	if (ctx == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	if (!EC_POINT_set_compressed_coordinates(group, point, bn_x,
-						 y_bit, ctx)) {
-		rc = -EIO;
-		goto out;
-	}
-
-	if (!EC_POINT_is_on_curve(group, point, ctx)) {
-		rc = -EIO;
-		goto out;
-	}
-
-	if (!EC_POINT_get_affine_coordinates(group, point, bn_x, bn_y,
-					     ctx)) {
-		rc = -EIO;
-		goto out;
-	}
-
-	BN_bn2binpad(bn_y, y, prime_len);
-
-out:
-	if (ctx != NULL)
-		BN_CTX_free(ctx);
-	if (point != NULL)
-		EC_POINT_free(point);
-	if (group != NULL)
-		EC_GROUP_free(group);
-	if (bn_x != NULL)
-		BN_free(bn_x);
-	if (bn_y != NULL)
-		BN_free(bn_y);
-
-	return rc;
-}
-
-/**
- * Converts an ECC public key given by the nid and the x and y coordinates into
- * an OpenSSL PKEY.
- *
- * @param nid               the OpenSSL nid of the ECC curve used
- * @param prime_len         the length of the prime in bytes. This is also the
- *                          length of the x and y coordinates.
- * @param x                 the x coordinate as big endian binary number in
- *                          prime_len size
- * @param y                 the y coordinate as big endian binary number in
- *                          prime_len size
- * @param pkey              On return: A PKEY containing the ECC public key.
- *
- * @returns zero for success, a negative errno in case of an error:
- *          -EINVAL: a function parameter is invalid
- *          -ENOMEM: failed to allocate memory
- *          -EIO: OpenSSL failed to generate the PKEY
- *          -ENOENT: OpenSSL does not know/support the curve (nid)
- */
-int ecc_pub_key_as_pkey(int nid, size_t prime_len, const unsigned char *x,
-			const unsigned char *y, EVP_PKEY **pkey)
-{
-	BIGNUM *bn_x = NULL, *bn_y = NULL;
-	EC_GROUP *group = NULL;
-	EC_KEY *ec = NULL;
-	int rc;
-
-	if (pkey == NULL || x == NULL || y == NULL)
-		return -EINVAL;
-
-	*pkey = NULL;
-
-	bn_x = BN_bin2bn(x, prime_len, NULL);
-	bn_y = BN_bin2bn(y, prime_len, NULL);
-	if (bn_x == NULL || bn_y == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	group = EC_GROUP_new_by_curve_name(nid);
-	if (group == NULL) {
-		rc = -ENOENT;
-		goto out;
-	}
-
-	ec = EC_KEY_new();
-	if (ec == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	rc = EC_KEY_set_group(ec, group);
-	if (rc != 1) {
-		rc = -EIO;
-		goto out;
-	}
-
-	rc = EC_KEY_set_public_key_affine_coordinates(ec, bn_x, bn_y);
-	if (rc != 1) {
-		rc = -EIO;
-		goto out;
-	}
-
-	*pkey = EVP_PKEY_new();
-	if (*pkey == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	rc = EVP_PKEY_assign_EC_KEY(*pkey, ec);
-	if (rc != 1) {
-		rc = -EIO;
-		goto out;
-	}
-
-	rc = 0;
-out:
-	if (bn_x != NULL)
-		BN_free(bn_x);
-	if (bn_y != NULL)
-		BN_free(bn_y);
-	if (group != NULL)
-		EC_GROUP_free(group);
-	if (rc != 0 && *pkey != NULL) {
-		EVP_PKEY_free(*pkey);
-		*pkey = NULL;
-	}
-	return rc;
-}
-
-/**
- * Converts an RSA public key given by the modulus and public exponent into
- * an OpenSSL PKEY.
- *
- * @param modulus           the modulus as big endian number
- * @param modulus_length    the length of the modulus in bytes
- * @param pub_exp           the public exponent as big endian number
- * @param pub_exp_length    the length of the public exponent in bytes
- * @param pkey_type         the PKEY type (EVP_PKEY_RSA or EVP_PKEY_RSA_PSS)
- * @param pkey              On return: A PKEY containing the RSA public key.
- *
- * @returns zero for success, a negative errno in case of an error:
- *          -EINVAL: a function parameter is invalid
- *          -ENOMEM: failed to allocate memory
- *          -EIO: OpenSSL failed to generate the PKEY
- */
-int rsa_pub_key_as_pkey(const unsigned char *modulus, size_t modulus_length,
-			const unsigned char *pub_exp, size_t pub_exp_length,
-			int pkey_type, EVP_PKEY **pkey)
-{
-	BIGNUM *bn_modulus = NULL, *bn_pub_exp = NULL;
-	RSA *rsa;
-	int rc;
-
-	if (pkey == NULL || modulus == NULL || pub_exp == NULL)
-		return -EINVAL;
-
-	if (pkey_type != EVP_PKEY_RSA && pkey_type != EVP_PKEY_RSA_PSS)
-		return -EINVAL;
-
-	*pkey = NULL;
-
-	bn_modulus = BN_bin2bn(modulus, modulus_length, NULL);
-	bn_pub_exp = BN_bin2bn(pub_exp, pub_exp_length, NULL);
-	if (bn_modulus == NULL || bn_pub_exp == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	rsa = RSA_new();
-	if (rsa == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	rc = RSA_set0_key(rsa, bn_modulus, bn_pub_exp, NULL);
-	if (rc != 1) {
-		rc = -EIO;
-		goto out;
-	}
-
-	*pkey = EVP_PKEY_new();
-	if (*pkey == NULL) {
-		rc = -ENOMEM;
-		goto out;
-	}
-
-	rc = EVP_PKEY_assign(*pkey, pkey_type, rsa);
-	if (rc != 1) {
-		rc = -EIO;
-		goto out;
-	}
-
-	rc = 0;
-
-out:
-	if (rc != 0 && bn_modulus != NULL)
-		BN_free(bn_modulus);
-	if (rc != 0 && bn_pub_exp != NULL)
-		BN_free(bn_pub_exp);
-
-	return rc;
-}
-
 /**
  * Converts a JSON Web Key (ECC or RSA) into a OpenSSL PKEY
  *
@@ -2209,6 +1777,8 @@ out:
 int json_web_key_as_pkey(json_object *jwk, int pkey_type, EVP_PKEY **pkey)
 {
 	unsigned char *x = NULL, *y = NULL, *n = NULL, *e = NULL;
+	const struct sk_ec_curve_info *curve_info;
+	struct sk_pub_key_info pub_key = { 0 };
 	size_t prime_len, len, n_len, e_len;
 	const char *kty, *crv;
 	int nid, rc = 0;
@@ -2231,13 +1801,18 @@ int json_web_key_as_pkey(json_object *jwk, int pkey_type, EVP_PKEY **pkey)
 			goto out;
 		}
 
-		nid = ecc_get_curve_by_id(crv);
-		if (nid == 0) {
+		nid = EC_curve_nist2nid(crv);
+		if (nid == NID_undef) {
 			rc = -EIO;
 			goto out;
 		}
 
-		prime_len = ecc_get_curve_prime_length(nid);
+		curve_info = SK_UTIL_ec_get_curve_info(nid);
+		if (curve_info == NULL) {
+			rc = -EIO;
+			goto out;
+		}
+		prime_len = curve_info->prime_len;
 		if (prime_len == 0) {
 			rc = -EIO;
 			goto out;
@@ -2270,9 +1845,11 @@ int json_web_key_as_pkey(json_object *jwk, int pkey_type, EVP_PKEY **pkey)
 			goto out;
 		}
 
-		rc = ecc_pub_key_as_pkey(nid, prime_len, x, y, pkey);
-		if (rc != 0)
-			goto out;
+		pub_key.type = SK_KEY_TYPE_EC;
+		pub_key.ec.curve_nid = nid;
+		pub_key.ec.prime_len = prime_len;
+		pub_key.ec.x = x;
+		pub_key.ec.y = y;
 	} else if (strcmp(kty, "RSA") == 0) {
 		n_len = 0;
 		rc = json_object_get_base64url(jwk, "n", NULL, &n_len);
@@ -2304,14 +1881,21 @@ int json_web_key_as_pkey(json_object *jwk, int pkey_type, EVP_PKEY **pkey)
 		if (rc != 0)
 			goto out;
 
-		rc = rsa_pub_key_as_pkey(n, n_len, e, e_len, pkey_type, pkey);
-		if (rc != 0)
-			goto out;
-
+		pub_key.type = SK_KEY_TYPE_RSA;
+		pub_key.rsa.modulus = n;
+		pub_key.rsa.modulus_len = n_len;
+		pub_key.rsa.pub_exp = e;
+		pub_key.rsa.pub_exp_len = e_len;
 	} else {
 		return -EIO;
 	}
 
+	rc = SK_OPENSSL_get_pkey(NULL, 0, &pub_key,
+				 pkey_type == EVP_PKEY_RSA_PSS, NULL, NULL,
+				 pkey, false);
+	if (rc != 0)
+		goto out;
+
 out:
 	if (x != NULL)
 		free(x);
@@ -2583,521 +2167,6 @@ int write_public_key(const char *pem_filename, EVP_PKEY *pkey)
 	return 0;
 }
 
-/* Secure key PKEY context control */
-#define EVP_PKEY_CTRL_SK_KEY_BLOB		0x10000001
-#define EVP_PKEY_CTRL_SK_SIGN_FUNCTIONS		0x10000002
-#define EVP_PKEY_CTRL_SK_PRIVATE_DATA		0x10000003
-
-/* Secure key PKEY context data */
-struct sk_pkey_ctx {
-	rsa_sign_t         rsa_sign;
-	rsa_pss_sign_t     rsa_pss_sign;
-	ecdsa_sign_t       ecdsa_sign;
-
-	unsigned char      *key_blob;
-	size_t             key_blob_length;
-
-	const EVP_MD       *md;
-	int                rsa_padding;
-	int                rsa_pss_saltlen;
-	const EVP_MD       *rsa_pss_mgfmd;
-
-	void               *private;
-};
-
-/**
- * Initialize an secure key PKEY context
- *
- * @param ctx                the PKEY context to initialize
- *
- * @returns 1 for success, 0 in case of an error
- */
-static int sk_pkey_meth_init(EVP_PKEY_CTX *ctx)
-{
-	struct sk_pkey_ctx *sk_ctx;
-	EVP_PKEY *pkey;
-
-	pkey = EVP_PKEY_CTX_get0_pkey(ctx);
-	if (pkey == NULL)
-		return 0;
-
-	sk_ctx = OPENSSL_zalloc(sizeof(struct sk_pkey_ctx));
-	if (sk_ctx == NULL)
-		return 0;
-
-	if (EVP_PKEY_id(pkey) == EVP_PKEY_RSA_PSS) {
-		sk_ctx->rsa_pss_saltlen = RSA_PSS_SALTLEN_AUTO;
-		sk_ctx->rsa_padding = RSA_PKCS1_PSS_PADDING;
-	}
-	if (EVP_PKEY_id(pkey) == EVP_PKEY_RSA)
-		sk_ctx->rsa_padding = RSA_PKCS1_PADDING;
-
-	EVP_PKEY_CTX_set_data(ctx, sk_ctx);
-
-	return 1;
-}
-
-/**
- * Cleanup an secure key PKEY context
- *
- * @param ctx                the PKEY context to to clean
- */
-static void sk_pkey_meth_cleanup(EVP_PKEY_CTX *ctx)
-{
-	struct sk_pkey_ctx *sk_ctx;
-
-	sk_ctx = EVP_PKEY_CTX_get_data(ctx);
-	if (sk_ctx != NULL) {
-		OPENSSL_free(sk_ctx);
-		EVP_PKEY_CTX_set_data(ctx, NULL);
-	}
-}
-
-/**
- * Copy an secure key PKEY context
- *
- * @param dst                the source PKEY context to copy from
- * @param src                the destinationPKEY context to copy to
- *
- * @returns 1 for success, 0 in case of an error
- */
-static int sk_pkey_meth_copy(EVP_PKEY_CTX *dst, EVP_PKEY_CTX *src)
-{
-	struct sk_pkey_ctx *sk_ctx_src, *sk_ctx_dst;
-
-	if (sk_pkey_meth_init(dst) != 1)
-		return 0;
-
-	sk_ctx_src = EVP_PKEY_CTX_get_data(src);
-	if (sk_ctx_src == NULL)
-		return 0;
-
-	sk_ctx_dst = EVP_PKEY_CTX_get_data(dst);
-	if (sk_ctx_dst == NULL)
-		return 0;
-
-	sk_ctx_dst->rsa_sign = sk_ctx_src->rsa_sign;
-	sk_ctx_dst->rsa_pss_sign = sk_ctx_src->rsa_pss_sign;
-	sk_ctx_dst->ecdsa_sign = sk_ctx_src->ecdsa_sign;
-	sk_ctx_dst->key_blob = sk_ctx_src->key_blob;
-	sk_ctx_dst->key_blob_length = sk_ctx_src->key_blob_length;
-	sk_ctx_dst->md = sk_ctx_src->md;
-	sk_ctx_dst->rsa_pss_saltlen = sk_ctx_src->rsa_pss_saltlen;
-	sk_ctx_dst->rsa_padding = sk_ctx_src->rsa_padding;
-	sk_ctx_dst->rsa_pss_mgfmd = sk_ctx_src->rsa_pss_mgfmd;
-	sk_ctx_dst->private = sk_ctx_src->private;
-
-	return 1;
-}
-
-/**
- * Perform a sign operation with the secure key PKEY context.
- * The data to be signed has already been hashed.
- *
- * The key to perform the sign operation with is available in the PKEY context.
- * For this secure key case, the pkey as well as the secure key blob is
- * available.
- *
- * @param ctx                the PKEY context to sign with
- * @param sig                the buffer to store the signature
- * @param siglen             On input: the size of the signature buffer
- *                           On return: the size of the signature
- * @param tbs                the data to be signed
- * @param tbslen             the size of the data to be signed
- *
- * @returns 1 for success, 0 in case of an error
- */
-static int sk_pkey_meth_sign(EVP_PKEY_CTX *ctx, unsigned char *sig,
-			     size_t *siglen, const unsigned char *tbs,
-			     size_t tbslen)
-{
-	int rc, md_type, mgf_md_type, saltlen, max_saltlen, hlen;
-	struct sk_pkey_ctx *sk_ctx;
-	EVP_PKEY *pkey;
-
-	sk_ctx = EVP_PKEY_CTX_get_data(ctx);
-	if (sk_ctx == NULL)
-		return 0;
-
-	pkey = EVP_PKEY_CTX_get0_pkey(ctx);
-	if (pkey == NULL)
-		return 0;
-
-	md_type = sk_ctx->md != NULL ? EVP_MD_type(sk_ctx->md) : NID_sha1;
-
-	switch (EVP_PKEY_id(pkey)) {
-	case EVP_PKEY_RSA:
-		if (sk_ctx->rsa_sign == NULL)
-			return 0;
-		rc = sk_ctx->rsa_sign(sk_ctx->key_blob, sk_ctx->key_blob_length,
-				      sig, siglen, tbs, tbslen,
-				      sk_ctx->rsa_padding, md_type,
-				      sk_ctx->private);
-		break;
-
-	case EVP_PKEY_RSA_PSS:
-		if (sk_ctx->rsa_pss_sign == NULL)
-			return 0;
-
-		if (sk_ctx->md != NULL)
-			hlen = EVP_MD_size(sk_ctx->md);
-		else
-			hlen = SHA_DIGEST_LENGTH;
-
-		if (sk_ctx->rsa_pss_mgfmd != NULL) {
-			mgf_md_type = EVP_MD_type(sk_ctx->rsa_pss_mgfmd);
-			hlen = EVP_MD_size(sk_ctx->rsa_pss_mgfmd);
-		} else {
-			mgf_md_type = md_type;
-		}
-
-		/*
-		 * We should be using RSA_bits(EVP_PKEY_get0_RSA(pkey)) here,
-		 * but EVP_PKEY_get0_RSA(pkey) does not work with PKEY/type
-		 * EVP_PKEY_RSA_PSS on older OpenSSL versions, so we fall back
-		 * on EVP_PKEY_bits in this case.
-		 */
-		max_saltlen = (EVP_PKEY_get0_RSA(pkey) != NULL ?
-					RSA_bits(EVP_PKEY_get0_RSA(pkey)) :
-					EVP_PKEY_bits(pkey)) / 8 - hlen - 2;
-
-		switch (sk_ctx->rsa_pss_saltlen) {
-		case RSA_PSS_SALTLEN_DIGEST:
-			saltlen = hlen;
-			break;
-		case RSA_PSS_SALTLEN_AUTO:
-		case RSA_PSS_SALTLEN_MAX:
-			saltlen = max_saltlen;
-			break;
-		default:
-			saltlen = sk_ctx->rsa_pss_saltlen;
-			break;
-		}
-
-		if (saltlen > max_saltlen || saltlen < 0)
-			return 0;
-
-		rc = sk_ctx->rsa_pss_sign(sk_ctx->key_blob,
-					  sk_ctx->key_blob_length,
-					  sig, siglen, tbs, tbslen,
-					  md_type, mgf_md_type, saltlen,
-					  sk_ctx->private);
-		break;
-
-	case EVP_PKEY_EC:
-		if (sk_ctx->ecdsa_sign == NULL)
-			return 0;
-
-		rc = sk_ctx->ecdsa_sign(sk_ctx->key_blob,
-					sk_ctx->key_blob_length, sig, siglen,
-					tbs, tbslen, md_type, sk_ctx->private);
-		break;
-
-	default:
-		rc = -1;
-	}
-
-	return rc == 0 ? 1 : 0;
-}
-
-/**
- * Control options of the secure key PKEY context
- *
- * Besides some standard controls, the following secure key PKEY context
- * specific controls are available:
- *   EVP_PKEY_CTRL_SK_KEY_BLOB         set the secure key blob into the context
- *   EVP_PKEY_CTRL_SK_SIGN_FUNCTIONS   set the sign functions for the context
- *   EVP_PKEY_CTRL_SK_PRIVATE_DATA     set the private data to the context
- *
- * @param ctx                the PKEY context to control
- * @param type               the control type
- * @param p1                 an integer option
- * @param p2                 a pointer option
- *
- * @returns 1 for success, 0 in case of an error, -2 if the control is not
- * supported by the context
- */
-static int sk_pkey_meth_ctrl(EVP_PKEY_CTX *ctx, int type, int p1, void *p2)
-{
-	struct sk_pkey_sign_func *func;
-	struct sk_pkey_ctx *sk_ctx;
-	EVP_PKEY *pkey;
-	int md_type;
-
-	sk_ctx = EVP_PKEY_CTX_get_data(ctx);
-	if (sk_ctx == NULL)
-		return 0;
-
-	pkey = EVP_PKEY_CTX_get0_pkey(ctx);
-	if (pkey == NULL)
-		return 0;
-
-	switch (type) {
-	case EVP_PKEY_CTRL_SK_KEY_BLOB:
-		sk_ctx->key_blob = p2;
-		sk_ctx->key_blob_length = p1;
-		break;
-
-	case EVP_PKEY_CTRL_SK_SIGN_FUNCTIONS:
-		func = (struct sk_pkey_sign_func *)p2;
-		sk_ctx->rsa_sign = func->rsa_sign;
-		sk_ctx->rsa_pss_sign = func->rsa_pss_sign;
-		sk_ctx->ecdsa_sign = func->ecdsa_sign;
-		break;
-
-	case EVP_PKEY_CTRL_SK_PRIVATE_DATA:
-		sk_ctx->private = p2;
-		break;
-
-	case EVP_PKEY_CTRL_MD:
-		md_type = EVP_MD_type((const EVP_MD *)p2);
-		if (md_type != NID_sha1 &&
-		    md_type != NID_ecdsa_with_SHA1 &&
-		    md_type != NID_sha224 &&
-		    md_type != NID_sha256 &&
-		    md_type != NID_sha384 &&
-		    md_type != NID_sha512 &&
-		    md_type != NID_sha3_224 &&
-		    md_type != NID_sha3_256 &&
-		    md_type != NID_sha3_384 &&
-		    md_type != NID_sha3_512)
-			return 0;
-		sk_ctx->md = p2;
-		break;
-
-	case EVP_PKEY_CTRL_GET_MD:
-		*(const EVP_MD **)p2 = sk_ctx->md;
-		break;
-
-	case EVP_PKEY_CTRL_RSA_PADDING:
-		switch (EVP_PKEY_id(pkey)) {
-		case EVP_PKEY_EC:
-			return -2;
-		case EVP_PKEY_RSA:
-			break;
-		case EVP_PKEY_RSA_PSS:
-			if (p1 != RSA_PKCS1_PSS_PADDING)
-				return -2;
-			break;
-		}
-		sk_ctx->rsa_padding = p1;
-		break;
-
-	case EVP_PKEY_CTRL_GET_RSA_PADDING:
-		switch (EVP_PKEY_id(pkey)) {
-		case EVP_PKEY_EC:
-			return -1;
-		case EVP_PKEY_RSA:
-		case EVP_PKEY_RSA_PSS:
-			*(int *)p2 = sk_ctx->rsa_padding;
-			break;
-		}
-		break;
-
-	case EVP_PKEY_CTRL_RSA_PSS_SALTLEN:
-		if (sk_ctx->rsa_padding != RSA_PKCS1_PSS_PADDING)
-			return -2;
-		if (p1 < RSA_PSS_SALTLEN_MAX)
-			return -2;
-		if (p1 == RSA_PSS_SALTLEN_MAX)
-			sk_ctx->rsa_pss_saltlen = RSA_PSS_SALTLEN_MAX_SIGN;
-		else
-			sk_ctx->rsa_pss_saltlen = p1;
-		break;
-
-	case EVP_PKEY_CTRL_GET_RSA_PSS_SALTLEN:
-		if (sk_ctx->rsa_padding != RSA_PKCS1_PSS_PADDING)
-			return -2;
-		*(int *)p2 = sk_ctx->rsa_pss_saltlen;
-		break;
-
-	case EVP_PKEY_CTRL_RSA_MGF1_MD:
-		if (sk_ctx->rsa_padding != RSA_PKCS1_PSS_PADDING)
-			return -2;
-		sk_ctx->rsa_pss_mgfmd = p2;
-		break;
-
-	case EVP_PKEY_CTRL_GET_RSA_MGF1_MD:
-		if (sk_ctx->rsa_padding != RSA_PKCS1_PSS_PADDING)
-			return -2;
-		*(const EVP_MD **)p2 = sk_ctx->rsa_pss_mgfmd != NULL ?
-					sk_ctx->rsa_pss_mgfmd : sk_ctx->md;
-		break;
-
-	case EVP_PKEY_CTRL_DIGESTINIT:
-	case EVP_PKEY_CTRL_PEER_KEY:
-	case EVP_PKEY_CTRL_PKCS7_SIGN:
-	case EVP_PKEY_CTRL_CMS_SIGN:
-		break;
-
-	default:
-		return -2;
-	}
-
-	return 1;
-}
-
-/**
- * Sets up a secure key PKEY method to handle PKEY sign operations of the
- * specified PKEY id (type) using secure key functions, instead of the default
- * ones.
- *
- * Note: This should be done only right before the sign operation is used, and
- * should be cleaned up using cleanup_secure_key_pkey_method() right after the
- * sign operation is finished, to not interfere with other PKEY usage.
- *
- * @param pkey_id            the PKEY id (type) to setup special handling
- *                           for. Use EVP_PKEY_id() to get the id of an existing
- *                           pkey.
- *
- * @returns zero for success, a negative errno in case of an error:
- *          -ENOMEM: failed to allocate memory
- *          -EIO: OpenSSL failed to setup the method
- */
-int setup_secure_key_pkey_method(int pkey_id)
-{
-	EVP_PKEY_METHOD *pkey_meth;
-
-	pkey_meth = EVP_PKEY_meth_new(pkey_id, 0);
-	if (pkey_meth == NULL)
-		return -ENOMEM;
-
-	EVP_PKEY_meth_set_init(pkey_meth, sk_pkey_meth_init);
-	EVP_PKEY_meth_set_cleanup(pkey_meth, sk_pkey_meth_cleanup);
-	EVP_PKEY_meth_set_copy(pkey_meth, sk_pkey_meth_copy);
-	EVP_PKEY_meth_set_ctrl(pkey_meth, sk_pkey_meth_ctrl, NULL);
-	EVP_PKEY_meth_set_sign(pkey_meth, NULL, sk_pkey_meth_sign);
-
-	if (EVP_PKEY_meth_add0(pkey_meth) != 1)
-		return -EIO;
-
-	return 0;
-}
-
-/**
- * Cleans up the secure key PKEY method to handle PKEY sign operations of the
- * specified PKEY id (type) using secure key functions.
- *
- * Note: Only call this function of you have previously setup the PKEY method
- *       using setup_secure_key_pkey_method().
- *
- * @param pkey_id            the PKEY id (type) to setup special handling
- *                           for. Use EVP_PKEY_id() to get the id of an existing
- *                           pkey.
- *
- * @returns zero for success, a negative errno in case of an error:
- *          -ENOENT: method not found
- *          -EIO: OpenSSL failed to cleanup the method
- */
-int cleanup_secure_key_pkey_method(int pkey_id)
-{
-	const EVP_PKEY_METHOD *pkey_meth;
-
-	pkey_meth = EVP_PKEY_meth_find(pkey_id);
-	if (pkey_meth == NULL)
-		return -ENOENT;
-
-	if (EVP_PKEY_meth_remove(pkey_meth) != 1)
-		return -EIO;
-
-	EVP_PKEY_meth_free((EVP_PKEY_METHOD *)pkey_meth);
-
-	return 0;
-}
-
-/**
- * Sets up a secure key PKEY context to handle PKEY sign operations using the
- * specified secure key blob, and secure key sign functions. The secure key blob
- * must match the PKEY that was used to create the context with. The pkey
- * contains the public key parts only, the secure key blob contains the
- * private (and possibly public) key parts of the same key.
- *
- * Note: Only call this function of you have previously setup the PKEY method
- *       using setup_secure_key_pkey_method().
- *
- * @param pkey_ctx           the PKEY context to setup special handling for
- * @param key_blob           the secure key blob to sign with
- * @param key_blob_len       the size of the key bob in bytes
- * @param sign_funcs         secure key specific sign functions
- * @param private            a pointer passed as is to the sign functions
- *
- * @returns zero for success, a negative errno in case of an error:
- *          -EINVAL: invalid argument
- *          -EIO: OpenSSL failed to setup the context
- */
-int setup_secure_key_pkey_context(EVP_PKEY_CTX *pkey_ctx,
-				  const unsigned char *key_blob,
-				  size_t key_blob_len,
-				  struct sk_pkey_sign_func *sign_funcs,
-				  void *private)
-{
-	int rc;
-
-	if (pkey_ctx == NULL || sign_funcs == NULL)
-		return -EINVAL;
-
-	rc = EVP_PKEY_CTX_ctrl(pkey_ctx, -1, -1, EVP_PKEY_CTRL_SK_KEY_BLOB,
-			       key_blob_len, (void *)key_blob);
-	if (rc != 1)
-		return -EIO;
-
-	rc = EVP_PKEY_CTX_ctrl(pkey_ctx, -1, -1,
-			       EVP_PKEY_CTRL_SK_SIGN_FUNCTIONS,
-			       0, sign_funcs);
-	if (rc != 1)
-		return -EIO;
-
-	rc = EVP_PKEY_CTX_ctrl(pkey_ctx, -1, -1, EVP_PKEY_CTRL_SK_PRIVATE_DATA,
-			       0, private);
-	if (rc != 1)
-		return -EIO;
-
-	return 0;
-}
-
-/**
- * Sets up a PKEY context with RSA-PSS parameters.
- *
- * @param pkey_ctx           the PKEY context to setup special handling for
- * @param rsa_pss_params     the RSA-PSS paramaters
- *
- * @returns zero for success, a negative errno in case of an error:
- *          -EINVAL: invalid argument
- *          -EIO: OpenSSL failed to setup the context
- */
-int setup_rsa_pss_pkey_context(EVP_PKEY_CTX *pkey_ctx,
-			       struct ekmf_rsa_pss_params *rsa_pss_params)
-{
-	const EVP_MD *mgf_md;
-	int rc;
-
-	if (pkey_ctx == NULL || rsa_pss_params == NULL)
-		return -EINVAL;
-
-	rc = EVP_PKEY_CTX_set_rsa_padding(pkey_ctx,
-					  RSA_PKCS1_PSS_PADDING);
-	if (rc != 1)
-		return -EIO;
-
-	rc = EVP_PKEY_CTX_set_rsa_pss_saltlen(pkey_ctx,
-					      rsa_pss_params->salt_len);
-	if (rc != 1)
-		return -EIO;
-
-	if (rsa_pss_params->mgf_digest_nid != 0) {
-		mgf_md = EVP_get_digestbynid(
-				rsa_pss_params->mgf_digest_nid);
-		if (mgf_md == NULL)
-			return -ENOENT;
-
-		rc = EVP_PKEY_CTX_set_rsa_mgf1_md(pkey_ctx, mgf_md);
-		if (rc != 1)
-			return -EIO;
-	}
-
-	return 0;
-}
-
 /**
  * Checks if an exact duplicate of the name entry is part of the name already.
  */
@@ -3230,12 +2299,12 @@ int build_subject_name(X509_NAME **name, const char *rdns[], size_t num_rdns,
 /**
  * Compares X509 Extensions by their nid
  */
-static int X509_EXTENSION_compfunc(const X509_EXTENSION * const* a,
-				   const X509_EXTENSION * const* b)
+static int X509_EXTENSION_compfunc(const X509_EXTENSION * const *a,
+				   const X509_EXTENSION * const *b)
 {
 
-	return (OBJ_obj2nid(X509_EXTENSION_get_object((X509_EXTENSION *)a)) -
-		OBJ_obj2nid(X509_EXTENSION_get_object((X509_EXTENSION *)b)));
+	return (OBJ_obj2nid(X509_EXTENSION_get_object((X509_EXTENSION *)*a)) -
+		OBJ_obj2nid(X509_EXTENSION_get_object((X509_EXTENSION *)*b)));
 }
 
 /**
