@@ -5,6 +5,9 @@
 mod create;
 pub use create::create;
 
+mod verify;
+pub use verify::verify;
+
 // Commands (directly) related to UVCs are only available on s389x
 #[cfg(target_arch = "s390x")]
 mod add;
