@@ -2,7 +2,7 @@
 #
 # dbginfo.sh - Tool to collect runtime, configuration, and trace information
 #
-# Copyright IBM Corp. 2002, 2023
+# Copyright IBM Corp. 2002, 2024
 #
 # s390-tools is free software; you can redistribute it and/or modify
 # it under the terms of the MIT license. See LICENSE for details.
@@ -15,6 +15,7 @@ export LC_ALL
 ########################################
 # Global used variables
 readonly SCRIPTNAME="${0##*/}" # general name of this script
+readonly STARTDIR="$(pwd)" # save calling directory
 #
 readonly DATETIME="$(date +%Y-%m-%d-%H-%M-%S 2>/dev/null)"
 readonly DOCKER=$(if type docker >/dev/null 2>&1; then echo "YES"; else echo "NO"; fi)
@@ -51,6 +52,14 @@ if test "x${PROCESSORVERSION}" = "xFF" || test "x${PROCESSORVERSION}" = "xff"; t
 else
 	RUNTIME_ENVIRONMENT="LPAR"
 fi
+# check for Dynamic Partition Mode
+readonly DPM_UUID=$(grep "LPAR UUID" /proc/sysinfo | \
+	sed 's/.*:[[:space:]]*\([[:graph:]]*\).*/\1/g')
+if test "${#DPM_UUID}" -eq 0; then
+	DPM_MODE="NO"
+else
+	DPM_MODE="YES"
+fi
 readonly SYSTEMHOSTNAME="$(hostname -s 2>/dev/null)" # hostname of system being analysed
 readonly TERMINAL="$(tty 2>/dev/null)"
 # timeout seconds TOS / kill timeout TOKS
@@ -72,7 +81,7 @@ paramWORKDIR_BASE="/tmp"  # initial default path
 print_version() {
 	cat <<EOF
 ${SCRIPTNAME}: Debug information script version %S390_TOOLS_VERSION%
-Copyright IBM Corp. 2002, 2023
+Copyright IBM Corp. 2002, 2024
 EOF
 }
 
@@ -155,7 +164,7 @@ print_check() {
 	cat <<EOF
 
 Hardware platform     = ${HW}
-Runtime environment   = ${RUNTIME_ENVIRONMENT}
+Runtime environment   = ${RUNTIME_ENVIRONMENT} - DPM: ${DPM_MODE}
 $(cat /proc/sysinfo | grep 'Name')
 Kernel version        = ${KERNEL_INFO}
 OS version / distro   = ${OS_NAME}
@@ -274,6 +283,7 @@ ALL_STEPS="\
   collect_sysfs\
   collect_procfs\
   collect_configfiles\
+  collect_initrd_configfiles\
   collect_cmdsout\
   collect_hyptop\
   collect_vmcmdsout\
@@ -381,14 +391,19 @@ CONFIGFILES="\
   /boot/loader/entries/*.conf\
   /boot/zipl/active_devices.txt\
   /boot/zipl/config\
+  /conf/\
   /etc/*.conf\
   /etc/*release\
   /etc/anacrontab\
   /etc/apparmor.d\
   /etc/auto.*\
+  /etc/cmdline\
+  /etc/cmdline.d\
+  /etc/conf.d\
   /etc/cron.*\
   /etc/crontab\
   /etc/crypttab\
+  /etc/dasd.*\
   /etc/default\
   /etc/depmod.d\
   /etc/dnf/dnf.conf\
@@ -429,12 +444,14 @@ CONFIGFILES="\
   /etc/systemd\
   /etc/udev*\
   /etc/xinet.d\
+  /etc/zfcp.*\
   /lib/modprobe.d\
   $(find /lib/modules -name modules.dep 2>/dev/null)\
   /lib/systemd/system/docker.service\
+  /lib/udev/rules.d\
   /usr/lib/modprobe.d\
   /usr/lib/modules-load.d/*.conf\
-  /usr/lib/systemd/system\
+  /usr/lib/systemd/system/\
   /usr/local/lib/modprobe.d\
   /usr/local/lib/modules-load.d/*.conf\
   /run/modprobe.d\
@@ -442,6 +459,7 @@ CONFIGFILES="\
   /run/udev/chreiplzfcpmp-ipl-volume-id\
   /run/udev/rules.d\
   /run/zdev_id.env\
+  /run/zdev.*\
   "
 
 ########################################
@@ -904,6 +922,89 @@ collect_configfiles() {
 	done
 }
 
+########################################
+collect_initrd_configfiles() {
+	local file_name
+	local rd rdbase rdreldir rdextract mypopd noglob mycat patterns=""
+
+	pr_collect "initrd config files"
+	cd "${WORKPATH}" || return
+	mypopd="$OLDPWD" # (pushd alternative)
+	# prevent expansion of glob special characters such as '*'
+	noglob=$(set +o | grep -F noglob)
+	set -o noglob
+	# prepare relative configfiles paths
+	for file_name in ${CONFIGFILES}; do
+	    # strip leading slash to make path relative as in initrd
+	    file_name=${file_name#/}
+	    # in contrast to call_collect_file(), cpio extract with patterns
+	    # does not recurse, but cpio wildcards match slash, so add suffix;
+	    # otherwise e.g. /lib/udev/rules.d or /etc/multipath only extracts
+	    # just that directory without any of its content
+	    file_name="${file_name}*"
+	    patterns="$patterns ${file_name}"
+	done
+	patterns="$patterns squash-root.img"
+	$noglob
+	# iterate over initrds
+	for rd in /boot/initr* /boot/zipl/initr* /var/lib/kdump/initr*; do
+	    [ -f "$rd" ] || continue # (nullglob alternative)
+	    rdbase=${rd##*/} # basename
+	    rdreldir=${rd%/*} # dirname
+	    rdreldir=${rdreldir#/} # strip leading slash to make path relative
+	    mkdir -p "${rdreldir}"
+	    cd "${rdreldir}" || break # (pushd alternative)
+	    if type lsinitrd >/dev/null 2>&1; then
+		lsinitrd "${rd}" > "${rdbase}.lsinitrd"
+	    fi
+	    if type lsinitramfs >/dev/null 2>&1; then
+		lsinitramfs -l "${rd}" > "${rdbase}.lsinitramfs"
+	    fi
+	    cd - || break # change to $OLDPWD (popd alternative)
+	    rdextract=${rd#/}
+	    mkdir -p "${rdextract}"
+	    cd "${rdextract}" || break # (pushd alternative)
+	    case $(file "${rd}") in
+		*cpio*) mycat="cat" ;;
+		*gzip*) mycat="zcat" ;;
+		*XZ*) mycat="xzcat" ;;
+		*Zstandard*) mycat="zstdcat" ;;
+		*bzip2*) mycat="bzcat" ;;
+		*LZMA*) mycat="lzcat" ;;
+		*)
+		    # change to $OLDPWD (popd alternative)
+		    cd - || { cd "$mypopd" || return; return; }
+		    continue
+		    ;;
+	    esac
+	    # prevent expansion of glob special characters such as '*'
+	    noglob=$(set +o | grep -F noglob)
+	    set -o noglob
+	    # we expect no early-initrd stuff on s390 so extract directly.
+	    # globbing is turned off and we want word split on $patterns:
+	    # shellcheck disable=SC2086
+	    "$mycat" "${rd}" | cpio --extract --no-absolute-filenames \
+				    --make-directories \
+				    --preserve-modification-time $patterns
+	    $noglob
+	    if [ ! -f "squash-root.img" ]; then
+		cd - || break # change to $OLDPWD (popd alternative)
+		continue
+	    fi
+	    # prevent expansion of glob special characters such as '*'
+	    noglob=$(set +o | grep -F noglob)
+	    set -o noglob
+	    # globbing is turned off and we want word split on $patterns:
+	    # shellcheck disable=SC2086
+	    unsquashfs -no-progress -dest "squashfs-root" "squash-root.img" \
+		       $patterns
+	    $noglob
+	    rm -f "squash-root.img"
+	    cd - || break # change to $OLDPWD (popd alternative)
+	done
+	cd "$mypopd" || return
+}
+
 ########################################
 collect_osaoat() {
 	local network_devices
@@ -1319,10 +1420,11 @@ environment_setup() {
 create_package() {
 	local rc_tar
 	pr_syslog_stdout ${step_num} "Finalizing: Creating archive with collected data"
+	# get a copy of the script used - enabled for relative path calls
+	cd "${STARTDIR}"
+	cp -p "${BASH_SOURCE[0]}" "${WORKPATH}"
+	# create the archive
 	cd "${WORKDIR_BASE}"
-	# get a copy of the script used
-	cp -p $0 "${WORKDIR_CURRENT}"
-
 	touch "${WORKARCHIVE}"
 	chmod 0600 "${WORKARCHIVE}"
 	tar -czf "${WORKARCHIVE}" "${WORKDIR_CURRENT}"
@@ -1437,7 +1539,7 @@ trap emergency_exit SIGHUP SIGINT SIGTERM
 
 pr_log_stdout ""
 pr_log_stdout "Hardware platform     = ${HW}"
-pr_log_stdout "Runtime environment   = ${RUNTIME_ENVIRONMENT}"
+pr_log_stdout "Runtime environment   = ${RUNTIME_ENVIRONMENT} - DPM: ${DPM_MODE}"
 pr_log_stdout "Kernel version        = ${KERNEL_INFO} (${KERNEL_BASE})"
 pr_log_stdout "OS version / distro   = ${OS_NAME}"
 pr_log_stdout "Date and time of info = ${DATETIME}"
