@@ -1,7 +1,7 @@
 /*
  * zcryptstats - Show usage statistics of IBM Crypto Express adapters
  *
- * Copyright IBM Corp. 2019
+ * Copyright IBM Corp. 2019, 2022
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -148,8 +148,9 @@ struct chsc_cmb_area {
 #define CRYPTO_TYPE_CEX5S	11
 #define CRYPTO_TYPE_CEX6S	12
 #define CRYPTO_TYPE_CEX7S	13
+#define CRYPTO_TYPE_CEX8S	14
 
-#define CRYPTO_TYPE_TOLERATION	CRYPTO_TYPE_CEX7S
+#define CRYPTO_TYPE_TOLERATION	CRYPTO_TYPE_CEX8S
 
 struct crypto_counter {
 	const char *name;
@@ -170,13 +171,13 @@ struct crypto_type {
 };
 
 #define NUM_COPROC_COUNTERS	2
-const struct crypto_counter counter_coproc[NUM_COPROC_COUNTERS] = {
+static const struct crypto_counter counter_coproc[NUM_COPROC_COUNTERS] = {
 		{ .name = "All", .is_totals = true },
 		{ .name = "RSA Key-gen" },
 };
 
 #define NUM_ACCEL_COUNTERS	6
-const struct crypto_counter counter_accel[NUM_ACCEL_COUNTERS] = {
+static const struct crypto_counter counter_accel[NUM_ACCEL_COUNTERS] = {
 		{ .name = "RSA 1024 ME" },
 		{ .name = "RSA 2048 ME" },
 		{ .name = "RSA 1024 CRT" },
@@ -186,7 +187,7 @@ const struct crypto_counter counter_accel[NUM_ACCEL_COUNTERS] = {
 };
 
 #define NUM_EP11_COUNTERS	5
-const struct crypto_counter counter_ep11[NUM_EP11_COUNTERS] = {
+static const struct crypto_counter counter_ep11[NUM_EP11_COUNTERS] = {
 		{ .name = "Asym. Slow" },
 		{ .name = "Asym. Fast" },
 		{ .name = "Symm. Partial" },
@@ -195,7 +196,7 @@ const struct crypto_counter counter_ep11[NUM_EP11_COUNTERS] = {
 };
 
 #define NUM_PCICA_COUNTERS	20
-const struct crypto_counter counter_pcica[NUM_PCICA_COUNTERS] = {
+static const struct crypto_counter counter_pcica[NUM_PCICA_COUNTERS] = {
 		{ .name = "RSA 1024 ME (E0)" },
 		{ .name = "RSA 2048 ME (E0)" },
 		{ .name = "RSA 1024 CRT (E0)" },
@@ -219,25 +220,25 @@ const struct crypto_counter counter_pcica[NUM_PCICA_COUNTERS] = {
 };
 
 #define NUM_COPROC_MODES	1
-const struct crypto_mode mode_coproc[1] = {
+static const struct crypto_mode mode_coproc[1] = {
 		{ .num_counters = NUM_COPROC_COUNTERS,
 		  .counters = counter_coproc},
 };
 
 #define NUM_ACCEL_MODES		1
-const struct crypto_mode mode_accel[1] = {
+static const struct crypto_mode mode_accel[1] = {
 		{ .num_counters = NUM_ACCEL_COUNTERS,
 		  .counters = counter_accel },
 };
 
 #define NUM_PCICA_MODES		1
-const struct crypto_mode mode_pcica[1] = {
+static const struct crypto_mode mode_pcica[1] = {
 		{ .num_counters = NUM_PCICA_COUNTERS,
 		  .counters = counter_pcica },
 };
 
-#define NUM_CEX4567_MODES	11
-const struct crypto_mode mode_cex4567[NUM_CEX4567_MODES] = {
+#define NUM_CEX45678_MODES	11
+static const struct crypto_mode mode_cex45678[NUM_CEX45678_MODES] = {
 		{ 0 },
 		{ 0 },
 		{ 0 },
@@ -257,8 +258,8 @@ const struct crypto_mode mode_cex4567[NUM_CEX4567_MODES] = {
 		  .counters = counter_ep11 },
 };
 
-#define NUM_CRYPTO_TYPES	14
-const struct crypto_type crypto_types[NUM_CRYPTO_TYPES] = {
+#define NUM_CRYPTO_TYPES	15
+static const struct crypto_type crypto_types[NUM_CRYPTO_TYPES] = {
 		{ 0 },
 		{ 0 },
 		{ 0 },
@@ -276,14 +277,16 @@ const struct crypto_type crypto_types[NUM_CRYPTO_TYPES] = {
 		  .modes = mode_accel },
 		{ .name = "CEX3C", .num_modes = NUM_COPROC_MODES,
 		  .modes = mode_coproc },
-		{ .name = "CEX4", .num_modes = NUM_CEX4567_MODES,
-		  .modes = mode_cex4567 },
-		{ .name = "CEX5", .num_modes = NUM_CEX4567_MODES,
-		  .modes = mode_cex4567 },
-		{ .name = "CEX6", .num_modes = NUM_CEX4567_MODES,
-		  .modes = mode_cex4567 },
-		{ .name = "CEX7", .num_modes = NUM_CEX4567_MODES,
-		  .modes = mode_cex4567 },
+		{ .name = "CEX4", .num_modes = NUM_CEX45678_MODES,
+		  .modes = mode_cex45678 },
+		{ .name = "CEX5", .num_modes = NUM_CEX45678_MODES,
+		  .modes = mode_cex45678 },
+		{ .name = "CEX6", .num_modes = NUM_CEX45678_MODES,
+		  .modes = mode_cex45678 },
+		{ .name = "CEX7", .num_modes = NUM_CEX45678_MODES,
+		  .modes = mode_cex45678 },
+		{ .name = "CEX8", .num_modes = NUM_CEX45678_MODES,
+		  .modes = mode_cex45678 },
 };
 
 
@@ -430,7 +433,7 @@ static const struct print_func csv_print = {
 /*
  * Program configuration
  */
-const struct util_prg prg = {
+static const struct util_prg prg = {
 	.desc = "Display usage statistics of IBM Crypto Express adapters",
 	.args = "[DEVICE_IDS]",
 	.copyright_vec = {
@@ -1178,8 +1181,14 @@ static int get_apqn_measurement_data(uint8_t card)
 		scdmd_area.request.header.code = 0x102d;
 		scdmd_area.request.header.length =
 				sizeof(struct chsc_scdmd_request);
-		scdmd_area.request.first_drid.ap_index = card;
-		scdmd_area.request.first_drid.domain_index = g.min_domain;
+		if (scdmd_area.response.p) {
+			scdmd_area.request.first_drid =
+						scdmd_area.response.crid;
+		} else {
+			scdmd_area.request.first_drid.ap_index = card;
+			scdmd_area.request.first_drid.domain_index =
+								g.min_domain;
+		}
 		scdmd_area.request.last_drid.ap_index = card;
 		scdmd_area.request.last_drid.domain_index = g.max_domain;
 		scdmd_area.request.s = 1;
@@ -1217,10 +1226,6 @@ static int get_apqn_measurement_data(uint8_t card)
 		rc = process_apqn_measurement_data(&scdmd_area);
 		if (rc != 0)
 			break;
-
-		if (scdmd_area.response.p)
-			scdmd_area.request.first_drid =
-					scdmd_area.response.crid;
 	} while (scdmd_area.response.p);
 
 	return rc;
@@ -2422,4 +2427,3 @@ out:
 
 	return rc;
 }
-
