@@ -15,6 +15,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <ctype.h>
 
 #include "lib/util_path.h"
 #include "lib/zt_common.h"
@@ -44,6 +45,12 @@
 #include "udev.h"
 #include "zfcp_lun.h"
 
+/* current site-id in action. By default, operations are always on fallback
+ * site; We need the current site-id as global variable to avoid excessive
+ * diff of simple function parameter modifications to pass this value.
+ */
+int global_site_id = SITE_FALLBACK;
+
 /* Main program action. */
 typedef enum {
 	ACT_CONFIGURE,
@@ -73,6 +80,7 @@ struct options {
 	/* Actions */
 	unsigned int enable:1;
 	unsigned int deconfigure:1;
+	unsigned int deconfigure_all:1;
 	unsigned int list_attribs:1;
 	unsigned int list_types:1;
 	unsigned int help_attribs:1;
@@ -97,6 +105,7 @@ struct options {
 	unsigned int verbose:1;
 	unsigned int quiet:1;
 	unsigned int no_settle:1;
+	unsigned int site_id;
 };
 
 /* Makefile converts chzdev_usage.txt into C file which we include here. */
@@ -119,6 +128,7 @@ enum {
 	OPT_TYPE		= 't',
 	OPT_ENABLE		= 'e',
 	OPT_DECONFIGURE		= 'd',
+	OPT_DECONFIGURE_ALL	= 'D',
 	OPT_LIST_ATTRIBS	= 'l',
 	OPT_HELP_ATTRIBS	= 'H',
 	OPT_LIST_TYPES		= 'L',
@@ -140,43 +150,52 @@ enum {
 	OPT_QUIET		= 'q',
 	OPT_NO_SETTLE		= (OPT_ANONYMOUS_BASE+__COUNTER__),
 	OPT_AUTO_CONF		= (OPT_ANONYMOUS_BASE+__COUNTER__),
+	OPT_SITE		= 's',
 };
 
 static struct opts_conflict conflict_list[] = {
 	OPTS_CONFLICT(OPT_ENABLE,
 		      OPT_DECONFIGURE, OPT_LIST_ATTRIBS, OPT_LIST_TYPES,
-		      OPT_EXPORT, OPT_IMPORT, OPT_APPLY, OPT_TYPE, 0),
+		      OPT_EXPORT, OPT_IMPORT, OPT_APPLY, OPT_TYPE,
+		      OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_DECONFIGURE,
 		      OPT_LIST_ATTRIBS, OPT_HELP_ATTRIBS, OPT_LIST_TYPES,
 		      OPT_EXPORT, OPT_IMPORT, OPT_APPLY, OPT_REMOVE,
-		      OPT_REMOVE_ALL, OPT_TYPE, 0),
+		      OPT_REMOVE_ALL, OPT_TYPE,
+		      OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_LIST_ATTRIBS,
 		      OPT_DECONFIGURE, OPT_HELP_ATTRIBS, OPT_LIST_TYPES,
 		      OPT_EXPORT, OPT_IMPORT, OPT_APPLY, OPT_REMOVE,
 		      OPT_REMOVE_ALL, OPT_CONFIGURED, OPT_EXISTING, OPT_ONLINE,
 		      OPT_OFFLINE, OPT_BY_PATH, OPT_BY_NODE, OPT_BY_INTERFACE,
-		      OPT_BY_ATTRIB, OPT_ACTIVE, OPT_PERSISTENT, OPT_FAILED, 0),
+		      OPT_BY_ATTRIB, OPT_ACTIVE, OPT_PERSISTENT, OPT_FAILED,
+		      OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_LIST_TYPES,
 		      OPT_DECONFIGURE, OPT_LIST_ATTRIBS, OPT_HELP_ATTRIBS,
 		      OPT_EXPORT, OPT_IMPORT, OPT_APPLY, OPT_REMOVE,
 		      OPT_REMOVE_ALL, OPT_CONFIGURED, OPT_EXISTING, OPT_ONLINE,
 		      OPT_OFFLINE, OPT_BY_PATH, OPT_BY_NODE, OPT_BY_INTERFACE,
-		      OPT_BY_ATTRIB, OPT_ACTIVE, OPT_PERSISTENT, OPT_FAILED, 0),
+		      OPT_BY_ATTRIB, OPT_ACTIVE, OPT_PERSISTENT, OPT_FAILED,
+		      OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_EXPORT,
 		      OPT_DECONFIGURE, OPT_LIST_ATTRIBS, OPT_HELP_ATTRIBS,
 		      OPT_LIST_TYPES, OPT_IMPORT, OPT_APPLY, OPT_REMOVE,
-		      OPT_REMOVE_ALL, 0),
+		      OPT_REMOVE_ALL, OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_IMPORT,
 		      OPT_DECONFIGURE, OPT_LIST_ATTRIBS, OPT_HELP_ATTRIBS,
 		      OPT_LIST_TYPES, OPT_EXPORT, OPT_APPLY, OPT_REMOVE,
 		      OPT_REMOVE_ALL, OPT_CONFIGURED, OPT_EXISTING, OPT_BY_PATH,
-		      OPT_BY_ATTRIB, OPT_BY_NODE, OPT_BY_INTERFACE, 0),
+		      OPT_BY_ATTRIB, OPT_BY_NODE, OPT_BY_INTERFACE,
+		      OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_APPLY,
 		      OPT_DECONFIGURE, OPT_LIST_ATTRIBS, OPT_HELP_ATTRIBS,
 		      OPT_LIST_TYPES, OPT_EXPORT, OPT_IMPORT, OPT_REMOVE,
-		      OPT_REMOVE_ALL, OPT_ACTIVE, 0),
+		      OPT_REMOVE_ALL, OPT_ACTIVE,
+		      OPT_DECONFIGURE_ALL, 0),
 	OPTS_CONFLICT(OPT_ONLINE,
 		      OPT_OFFLINE),
+	OPTS_CONFLICT(OPT_SITE,
+		      OPT_TYPE, 0),
 	OPTS_CONFLICT(OPT_QUIET,
 		      OPT_VERBOSE),
 	OPTS_CONFLICT(OPT_AUTO_CONF,
@@ -202,6 +221,7 @@ static const struct option opt_list[] = {
 	/* Actions. */
 	{ "enable",		no_argument,	NULL, OPT_ENABLE },
 	{ "disable",		no_argument,	NULL, OPT_DECONFIGURE },
+	{ "disable-all",	no_argument,	NULL, OPT_DECONFIGURE_ALL },
 	{ "list-attributes",	no_argument,	NULL, OPT_LIST_ATTRIBS },
 	{ "help-attribute",	no_argument,	NULL, OPT_HELP_ATTRIBS },
 	{ "list-types",		no_argument,	NULL, OPT_LIST_TYPES },
@@ -225,11 +245,12 @@ static const struct option opt_list[] = {
 	{ "verbose",		no_argument,	NULL, OPT_VERBOSE },
 	{ "quiet",		no_argument,	NULL, OPT_QUIET },
 	{ "no-settle",		no_argument,	NULL, OPT_NO_SETTLE },
+	{ "site",		required_argument, NULL, OPT_SITE },
 	{ NULL,			no_argument,	NULL, 0 },
 };
 
 /* Command line abbreviations. */
-static const char opt_str[] = ":edlHLapr:RfyhvVqt";
+static const char opt_str[] = ":edlHLapr:RfyhvVqts:D";
 
 /* Count of persistently modified devices. */
 static int pers_mod_devs;
@@ -246,6 +267,8 @@ static void init_options(struct options *opts)
 	opts->settings = strlist_new();
 	opts->remove = strlist_new();
 	opts->base = strlist_new();
+	/* Default operations are on fallback site*/
+	opts->site_id = SITE_FALLBACK;
 }
 
 /* Release memory used in options data structure. */
@@ -838,6 +861,12 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 			opts->deconfigure = 1;
 			break;
 
+		case OPT_DECONFIGURE_ALL:
+			/* --disable all the settings*/
+			opts->deconfigure = 1;
+			opts->deconfigure_all = 1;
+			break;
+
 		case OPT_LIST_ATTRIBS:
 			/* --list-attributes */
 			opts->list_attribs = 1;
@@ -955,6 +984,28 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 			opts->no_settle = 1;
 			break;
 
+		case OPT_SITE:
+			/* --site */
+			/* 1. User can specify only site-ids from 0 to 9
+			 * 2. only one --site parameter is accepted
+			 */
+
+			if (!is_valid_site(optarg)) {
+				syntax("Unsupported site ID\n");
+				return EXIT_USAGE_ERROR;
+			}
+
+			if (opts->site_id != SITE_FALLBACK) {
+				syntax("Cannot specify '--site' multiple "
+				       "times\n");
+				return EXIT_USAGE_ERROR;
+			}
+
+			opts->site_id	= atoi(optarg);
+			global_site_id	= opts->site_id;
+			opts->persistent = 1;
+			break;
+
 		case ':':
 			/* Missing option argument. */
 			syntax("Option '%s' requires an argument\n",
@@ -987,7 +1038,9 @@ static exit_code_t parse_options(struct options *opts, int argc, char *argv[])
 		goto out;
 
 	/* Determine configuration set. */
-	if (!opts->active && !opts->persistent && !opts->auto_conf) {
+	if (opts->deconfigure_all) {
+		opts->config = config_all;
+	} else if (!opts->active && !opts->persistent && !opts->auto_conf) {
 		/* Default configuration targets are active + persistent */
 		opts->config = config_active | config_persistent;
 	} else {
@@ -1794,6 +1847,24 @@ static void unblacklist_ranges(struct selected_dev_node *sel,
 	*param_ptr = param;
 }
 
+/* Currently the site-specific configurations are supported only
+ * for some device types. Make sure that user get an error message when
+ * try to configure other device-types with site parameter.
+ */
+static exit_code_t is_site_supported(struct util_list *selected)
+{
+	struct selected_dev_node *sel;
+
+	util_list_iterate(selected, sel) {
+		if (sel->dt && !sel->dt->site_support) {
+			error("Site specific configuration is not supported"
+			      " for %s devices\n", sel->dt->devname);
+			return EXIT_USAGE_ERROR;
+		}
+	}
+	return EXIT_OK;
+}
+
 /* Handle device configuration. */
 static exit_code_t configure_devices(struct options *opts, int specified,
 				     int *found_ptr)
@@ -1827,6 +1898,9 @@ static exit_code_t configure_devices(struct options *opts, int specified,
 		error("No device was selected!\n");
 		rc = EXIT_EMPTY_SELECTION;
 		goto out;
+	} else  if (opts->site_id != SITE_FALLBACK) {
+		if (is_site_supported(selected))
+			goto out;
 	}
 
 	/* Work on selected devices. */
@@ -1913,6 +1987,18 @@ static exit_code_t check_in_use(struct device *dev)
 	return EXIT_OK;
 }
 
+static bool is_site_modified(struct device *dev)
+{
+	int i;
+
+	for (i = 0; i < NUM_USER_SITES; i++) {
+		if (dev->site_specific[i].modified)
+			return true;
+	}
+
+	return false;
+}
+
 /* Deconfigure specified device. */
 static exit_code_t deconfigure_one_device(struct subtype *st, const char *id,
 					  struct options *opts, int try,
@@ -1922,6 +2008,7 @@ static exit_code_t deconfigure_one_device(struct subtype *st, const char *id,
 	exit_code_t rc;
 	struct device *dev;
 	config_t config, read_config;
+	int i, site_available = 0;
 
 	/* Reset processed flag. */
 	if (proc_ptr)
@@ -1954,7 +2041,7 @@ static exit_code_t deconfigure_one_device(struct subtype *st, const char *id,
 	dev->processed = 1;
 
 	if (try && !(dev->active.exists || dev->active.definable ||
-		     dev->persistent.exists || dev->autoconf.exists)) {
+		     is_dev_pers(dev) || dev->autoconf.exists)) {
 		/* Attempt to deconfigure this device will fail. */
 		return EXIT_DEVICE_NOT_FOUND;
 	}
@@ -1964,7 +2051,7 @@ static exit_code_t deconfigure_one_device(struct subtype *st, const char *id,
 	 * dev. */
 	if (SCOPE_ACTIVE(config) &&
 	    !(dev->active.exists || dev->active.definable)) {
-		if (!((SCOPE_PERSISTENT(config) && dev->persistent.exists) ||
+		if (!((SCOPE_PERSISTENT(config) && is_dev_pers(dev)) ||
 		      (SCOPE_AUTOCONF(config) && dev->autoconf.exists)))
 			return EXIT_DEVICE_NOT_FOUND;
 	}
@@ -1985,16 +2072,48 @@ static exit_code_t deconfigure_one_device(struct subtype *st, const char *id,
 		} else if (subtype_online_get(st, dev, config_active) == 1)
 			dev->active.modified = 1;
 	}
-	if (SCOPE_PERSISTENT(config) && dev->persistent.exists) {
-		dev->persistent.deconfigured = 1;
-		dev->persistent.modified = 1;
+	if (SCOPE_PERSISTENT(config)) {
+		if (dev->persistent.exists) {
+			dev->persistent.deconfigured = 1;
+			dev->persistent.modified = 1;
+		} else if (is_dev_pers(dev) && global_site_id != SITE_FALLBACK) {
+			dev->persistent.deconfigured = 1;
+			dev->persistent.modified = 1;
+		}
 	}
 	if (SCOPE_AUTOCONF(config) && dev->autoconf.exists) {
 		dev->autoconf.deconfigured = 1;
 		dev->autoconf.modified = 1;
 	}
+
+	/* In case of deconfigure_all command, make sure that site specific
+	 * settings  are also removed.
+	 */
+	if (opts->deconfigure_all) {
+		for (i = 0; i < NUM_USER_SITES; i++) {
+			if (dev->site_specific[i].exists) {
+				site_available++;
+				/* To remove site-specific settings, simply
+				 * make the deconfigured = 1
+				 */
+				dev->site_specific[i].deconfigured = 1;
+				dev->site_specific[i].modified = 1;
+			}
+		}
+	}
+
+	if (site_available) {
+		/* set persistent.exists = 1 to let the
+		 * drivers know that we need to modify the
+		 * udev-rule
+		 */
+		dev->persistent.exists = 1;
+		dev->persistent.modified = 1;
+		dev->persistent.deconfigured = 1;
+	}
+
 	if (!dev->active.modified && !dev->persistent.modified &&
-	    !dev->autoconf.modified)
+	    !dev->autoconf.modified && !is_site_modified(dev))
 		return EXIT_OK;
 
 	/* Pre-write check. */
@@ -2036,6 +2155,9 @@ static exit_code_t deconfigure_devices(struct options *opts)
 		error("No device was selected!\n");
 		rc = EXIT_EMPTY_SELECTION;
 		goto out;
+	} else  if (opts->site_id != SITE_FALLBACK) {
+		if (is_site_supported(selected))
+			goto out;
 	}
 
 	/* Work on selected devices. */
@@ -2410,7 +2532,7 @@ static exit_code_t do_list_attribs(struct options *opts)
 	remove_duplicate_attribs(attribs, devtype_count_subtypes(dt));
 
 	if (attribs) {
-		table_attribs_show(attribs, 1, 0, dt);
+		table_attribs_show(attribs, 1, 0, 0, dt);
 		ptrlist_free(attribs, 1);
 	} else
 		rc = EXIT_ATTRIB_NOT_FOUND;
@@ -2558,8 +2680,13 @@ static void action_note(const char *msg, config_t config)
 {
 	if (config == config_active)
 		info("%s the active configuration only\n", msg);
-	else if (config == config_persistent)
-		info("%s the persistent configuration only\n", msg);
+	else if (config == config_persistent) {
+		if (global_site_id == SITE_FALLBACK)
+			info("%s the persistent configuration only\n", msg);
+		else
+			info("%s the site %d configuration only\n", msg,
+			     global_site_id);
+	}
 	else if (config == config_autoconf)
 		info("%s the auto-configuration only\n", msg);
 }
@@ -3034,7 +3161,7 @@ int main(int argc, char *argv[])
 	    !dryrun) {
 		/* If the root device/device type or early devices have been
 		 * modified, additional work might be necessary. */
-		rc = initrd_check(ZDEV_ALWAYS_UPDATE_INITRD);
+		rc = initrd_check(ZDEV_ALWAYS_UPDATE_INITRD, opts.deconfigure_all);
 		if (rc && !drc)
 			drc = rc;
 	}
