@@ -20,6 +20,7 @@ struct options {
 	char		*device;
 	/* If `output_path == NULL` the output is written to `stdout` */
 	const char	*output_path;
+	const char *key_path;
 	char		*mount_point;
 	int		fmt_specified;
 	const char	*fmt;
@@ -36,5 +37,7 @@ extern const char *OPTS_SELECT_PROD;
 extern const char *OPTS_SELECT_ALL;
 
 void opts_parse(int argc, char *argv[], struct options *opts);
+void opts_print_usage(const char *prog_name);
+void __noreturn print_usage_exit(const char *prog_name);
 
 #endif /* OPTS_H */
