@@ -26,7 +26,14 @@
 #include <time.h>
 #include <unistd.h>
 
+#include "lib/zt_common.h"
+#include "opts.h"
 #include "zgetdump.h"
+#include "dt.h"
+#include "dfi.h"
+#include "dfo.h"
+#include "output.h"
+#include "zfuse.h"
 
 /*
  * Globals
@@ -49,7 +56,7 @@ static void sig_exit(int sig)
  */
 static void sig_handler_init(void)
 {
-	struct sigaction sigact;
+	struct sigaction sigact = { 0 };
 
 	/* Ignore signals SIGUSR1 and SIGUSR2 */
 	if (sigemptyset(&sigact.sa_mask) < 0)
@@ -100,6 +107,20 @@ static void kdump_select_check(void)
 	ERR_EXIT("%s", msg);
 }
 
+static int dfi_init_error(int rc)
+{
+	if (rc == -ENOKEY) {
+		if (g.opts.key_path)
+			ERR("Cannot read key: '%s'", g.opts.key_path);
+		else
+			ERR("No key for dump decryption provided.");
+		opts_print_usage(g.opts.prog_name);
+	} else {
+		STDERR("Dump cannot be processed (is not complete)\n");
+	}
+	return 1;
+}
+
 /*
  * Run "--umount" action
  */
@@ -124,10 +145,13 @@ static int do_device_info(void)
  */
 static int do_dump_info(void)
 {
-	if (dfi_init() != 0) {
+	int rc;
+
+	rc = dfi_init();
+	if (rc != 0) {
 		dfi_info_print();
-		STDERR("\nERROR: Dump is not complete\n");
-		zg_exit(1);
+		STDERR("\n");
+		return dfi_init_error(rc);
 	}
 	kdump_select_check();
 	dfi_info_print();
@@ -142,8 +166,9 @@ static int do_mount(void)
 {
 	int rc;
 
-	if (dfi_init() != 0)
-		ERR_EXIT("Dump cannot be processed (is not complete)");
+	rc = dfi_init();
+	if (rc != 0)
+		return dfi_init_error(rc);
 	dfo_init();
 	kdump_select_check();
 	rc = zfuse_mount_dump();
@@ -151,18 +176,47 @@ static int do_mount(void)
 	return rc;
 }
 
-/*
- * Run "copy to stdout" action
- */
-static int do_stdout(void)
+static FILE *open_file_for_writing(const char *output)
 {
+	FILE *stream;
+	int fd;
+
+	if (output) {
+		fd = open(output,
+			  O_CREAT | O_EXCL | O_WRONLY | O_CLOEXEC,
+			  0600);
+		if (fd == -1)
+			ERR_EXIT_ERRNO("Could not open '%s' exclusively", output);
+	} else {
+		fd = dup(STDOUT_FILENO);
+		if (fd == -1)
+			ERR_EXIT_ERRNO("Could not dup() stdout");
+	}
+
+	stream = fdopen(fd, "w");
+	if (!stream)
+		ERR_EXIT_ERRNO("Could not fdopen()");
+	fd = -1;
+
+	return stream;
+}
+
+/*
+ * Run "copy to output" action
+ */
+static int do_copy(const char *output)
+{
+	FILE *stream;
 	int rc;
 
-	if (dfi_init() != 0)
-		ERR_EXIT("Dump cannot be processed (is not complete)");
+	rc = dfi_init();
+	if (rc != 0)
+		return dfi_init_error(rc);
 	dfo_init();
 	kdump_select_check();
-	rc = stdout_write_dump();
+	stream = open_file_for_writing(output);
+	rc = write_dump(stream);
+	fclose(stream);
 	dfi_exit();
 	return rc;
 }
@@ -173,11 +227,14 @@ static int do_stdout(void)
 int main(int argc, char *argv[])
 {
 	sig_handler_init();
-	opts_parse(argc, argv);
+	opts_parse(argc, argv, &g.opts);
+
+	if (dfo_set(g.opts.fmt) != 0)
+		ERR_EXIT("Invalid target format \"%s\" specified", g.opts.fmt);
 
 	switch (g.opts.action) {
-	case ZG_ACTION_STDOUT:
-		return do_stdout();
+	case ZG_ACTION_COPY:
+		return do_copy(g.opts.output_path);
 	case ZG_ACTION_DUMP_INFO:
 		return do_dump_info();
 	case ZG_ACTION_DEVICE_INFO:
