@@ -3,7 +3,7 @@
  *
  * Common ECKD dump I/O functions
  *
- * Copyright IBM Corp. 2013, 2018
+ * Copyright IBM Corp. 2013, 2023
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -11,6 +11,8 @@
 #ifndef ECKD2DUMP_H
 #define ECKD2DUMP_H
 
+#include "dump/s390_dump.h"
+
 #include "cio.h"
 #include "stage2dump.h"
 
@@ -49,5 +51,9 @@ void writeblock(unsigned long blk, unsigned long addr, unsigned long blk_count,
 void readblock(unsigned long blk, unsigned long addr, unsigned long blk_count);
 unsigned long write_dump_segment(unsigned long blk,
 				 struct df_s390_dump_segm_hdr *segm);
+#define NO_PROGRESS		0
+#define PRINT_PROGRESS		1
+unsigned long write_addr_range(unsigned long blk, unsigned long addr,
+			       unsigned long len, int print_progress);
 
 #endif /* ECKD2DUMP_H */
