@@ -17,10 +17,6 @@
 #include "lib/util_base.h"
 #include "boot/loaders_layout.h"
 
-#define ZIPL_MAGIC			"zIPL"
-#define ZIPL_MAGIC_SIZE			4
-#define DISK_LAYOUT_ID			0x00000001
-
 #define ADDRESS_LIMIT			0x80000000UL
 #define UNSPECIFIED_ADDRESS		-1UL
 #define LEGACY_MAXIMUM_PARMLINE_SIZE	0x380UL
@@ -29,6 +25,7 @@
 
 #define BOOTMAP_FILENAME		"bootmap"
 #define BOOTMAP_TEMPLATE_FILENAME	"bootmap_temp.XXXXXX"
+#define DUMP_TEMP_MOUNT_POINT_NAME	"zipl-dump-mount-point-XXXXXX"
 
 #define DEFAULTBOOT_SECTION		"defaultboot"
 
