@@ -25,7 +25,6 @@
 #include "lib/util_libc.h"
 #include "lib/util_part.h"
 #include "lib/util_path.h"
-#include "boot/s390.h"
 #include "stage3.h"
 
 #include "boot.h"
@@ -119,8 +118,6 @@ void bootmap_store_blockptr(void *buffer, disk_blockptr_t *ptr,
 	}
 }
 
-#define PROGRAM_TABLE_BLOCK_SIZE	512
-
 /* Calculate the maximum number of entries in the program table. INFO
  * specifies the type of disk. */
 static int
